package com.pj.current.netty.command.config;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.pj.current.config.MyConfig;
import com.pj.current.netty.command.factory.CommandFactory;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.project.tb_command_log.TbCommandLog;
import com.pj.project.tb_command_log.TbCommandLogService;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.utils.cache.RedisUtil;
import io.netty.channel.*;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@ChannelHandler.Sharable
@Component
public class CommandNettyServerHandler extends ChannelInboundHandlerAdapter {
    /**
     * 管理一个全局map，保存连接进服务端的通道数量
     */
    private static final ConcurrentHashMap<ChannelId, ChannelHandlerContext> CHANNEL_MAP = new ConcurrentHashMap<>();


    /**
     * @param ctx
     * @DESCRIPTION: 有客户端连接服务器会触发此函数
     * @return: void
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) {

    }

    /**
     * @param ctx
     * @DESCRIPTION: 有客户端终止连接服务器会触发此函数
     * @return: void
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) {
        log.info("设备离线======");
    }

    TimedCache<String, String> CONTENT_CACHE = CacheUtil.newTimedCache(30000);

    /**
     * @param ctx
     * @DESCRIPTION: 有客户端发消息会触发此函数
     * @return: void
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        String channelId = ctx.channel().id().asShortText();
        String str = msg.toString();
        if (!StrUtil.endWith(str, "}") && StrUtil.startWith(str, "{")) {
            CONTENT_CACHE.put(channelId, str);
            return;
        } else if (!StrUtil.endWith(str, "}") && !StrUtil.startWith(str, "{")) {
            String content = CONTENT_CACHE.get(channelId);
            content += str;
            CONTENT_CACHE.put(channelId, content);
            return;
        } else if (StrUtil.endWith(str, "}")) {
            String content = CONTENT_CACHE.get(channelId);
            if (StrUtil.isEmpty(content)) {
                content = "";
            }
            str = content + str;
            CONTENT_CACHE.remove(channelId);
        }
        log.info("终端请求:{}", str);
        JSONObject reqObj = JSONUtil.parseObj(str);
        Integer code = reqObj.getInt("code");
        if (code != null) {//设备端回复服务器的消息，无需再回复设备
            String msgId = RedisUtil.get(channelId);
            TbCommandLogService tbCommandLogService = SpringUtil.getBean(TbCommandLogService.class);
            TbCommandLog tbCommandLog = tbCommandLogService.findByMsgId(msgId);
            tbCommandLog.setResponseCode(code)
                    .setResponseContent(str)
                    .setResponseTime(new Date());
            tbCommandLogService.updateById(tbCommandLog);
            RedisUtil.del(channelId);
            return;
        }
        RequestPackageDTO packageDTO = JSONUtil.toBean(str, RequestPackageDTO.class);
        String funcId = packageDTO.getFuncId();
        CommandFactory commandFactory = SpringUtil.getBean(CommandFactory.class);
        commandFactory.handler(funcId).handler(ctx, packageDTO);

    }

    /**
     * @param msg 需要发送的消息内容
     * @DESCRIPTION: 服务端给客户端发送消息
     * @return: void
     */
    public void channelWrite(ChannelHandlerContext ctx, Object msg) throws Exception {
        //将客户端的信息直接返回写入ctx
        ctx.write(msg);
        //刷新缓存区
        ctx.flush();
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        String socketString = ctx.channel().remoteAddress().toString();

        if (evt instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) evt;
            if (event.state() == IdleState.READER_IDLE) {
                log.info("Client: " + socketString + " READER_IDLE 读超时");
                ctx.disconnect();
            } else if (event.state() == IdleState.WRITER_IDLE) {
                log.info("Client: " + socketString + " WRITER_IDLE 写超时");
                ctx.disconnect();
            } else if (event.state() == IdleState.ALL_IDLE) {
                log.info("Client: " + socketString + " ALL_IDLE 总超时");
                ctx.disconnect();
            }
        }
    }

    /**
     * @param ctx
     * @DESCRIPTION: 发生异常会触发此函数
     * @return: void
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ctx.close();
        log.info(ctx.channel().id() + " 发生了错误,此连接被关闭" + "此时连通数量: " + CHANNEL_MAP.size());
        cause.printStackTrace();
    }

}
